/*
------------------------------------------------------------------------------------------------------------------------------

   Auteur : A4 Technologie
   Date du programme : 25/07/2014
   Version : 1.0
   Application robotique : Portial Coulissant
   Référence du programme : Programme 7.3_corrigé Portail Coulissant

------------------------------------------------------------------------------------------------------------------------------

Configuration des Entrées :

Module                          	Nom de la Variable	Broche Arduino Associéé

Bouton-poussoir Intérieur	        bp_int	                          A0
Fin de course barrière ouverture 	fdc_ouverture	                  A1
Fin de course barrière fermeture 	fdc_fermeture	                  A2
Bouton-poussoir Extérieur	        bp_ext                            A3
Récepteur infrarouge	                recepteur_IR	                  A5




Configuration des Sorties :

Module                          	Nom de la Variable	Broche Arduino Associéé

Diode électroluminescente	        LED	                          4
Moteur sens ouverture	                ouverture	                  10=1 11=0
Moteur sens fermeture	                fermeture	                  10=0 11=1
Emetteur infrarouge	                emetteur_IR	                  5
Buzzer                                  buzzer                            8


*/

//Bibliothèque
#include <IRremote.h> // IMPORTANT, utiliser la bibliothèque distribuée par A4 Technologie

//Définition des variables, constantes et pin :

// Pin d'entrées
const int bp_ext = A3;
const int fdc_fermeture = A2;
const int fdc_ouverture = A1;
const int bp_int = A0;
const int recepteur_IR = A5;

//Pin de sorties
const int LED = 4;
const int emetteur_IR = 5;
const int buzzer = 8;

//Configuration moteur
const int ouverture = 10;
const int fermeture = 11;


//Initialisation recepteur_infrarouge
IRrecv irReciver(recepteur_IR);
decode_results results;
byte touche_appuyee; // variable

void setup()
{
  Serial.begin(9600); // ouvre le port série et paramètre le baudrate à 9600 bds

  //paramètrage des pins en entrée/sortie

  // Définition des pins comme des entrées
  pinMode (bp_ext, INPUT);
  pinMode (fdc_ouverture, INPUT);
  pinMode (fdc_fermeture, INPUT);
  pinMode (bp_int, INPUT);
  pinMode (recepteur_IR, INPUT);

  // Définition des pins comme des sorties
  pinMode (LED, OUTPUT);
  pinMode (ouverture, OUTPUT);
  pinMode (fermeture, OUTPUT);
  pinMode (emetteur_IR, OUTPUT);
  pinMode (buzzer, OUTPUT);

  // Etat Initial des sorties
  digitalWrite(ouverture, LOW); // Il est important que les deux états contrôlant le moteur
  digitalWrite(fermeture, LOW); // ne soient jamais tous les deux sur HIGH en même temps


  irReciver.enableIRIn(); // Début de la réception de la telecommande
}



//Programme principal

//Boucle infinie
void loop()
{

  // déclaration variable locale
  byte touche_appuyee;


  digitalWrite(emetteur_IR, LOW); // désactivation de l'emetteur IR pour recevoir le code de la télécommande
  while (touche_appuyee != 1) // rester dans la boucle tant que la touche appuyée n'est pas 1
  {
    touche_appuyee = fonction_telecommande(); // aller dans le sous-programme fonction_telecommande
  }


  digitalWrite(emetteur_IR, HIGH); // activation de l'emetteur IR pour réactiver la barrière IR
  delay(100); // temps pour initialiser l'emetteur infrarouge


  while (digitalRead(fdc_ouverture) == LOW) // rester dans la boucle tant que fdc_ouverture n'est pas activé
  {
    digitalWrite(ouverture, HIGH); // activation du moteur : sens ouverture
    clignotement_LED(); // aller dans le sous-programme clignotement_LED
    if (digitalRead(recepteur_IR) == HIGH) // si le camps infrarouge a été obstruit
    {
      securite();
      break; // sortie du programme
    }
  }

  arret();

  digitalWrite(emetteur_IR, LOW); // désactivation de l'emetteur IR pour recevoir le code de la télécommande

  while (touche_appuyee != 2) // rester dans la boucle tant que la touche appuyée n'est pas 1
  {
    touche_appuyee = fonction_telecommande(); // aller dans le sous-programme fonction_telecommande
  }


  digitalWrite(emetteur_IR, HIGH); // activation de l'emetteur IR
  delay(100); // temps pour initialiser l'emetteur infrarouge

  while (digitalRead(fdc_fermeture) == LOW) // condition sur l'état du capteur de fin de course fermeture
  {
    clignotement_LED();
    digitalWrite(fermeture, HIGH); // activation du moteur : sens fermeture
    if (digitalRead(recepteur_IR) == HIGH) // si le camps infrarouge a été obstruit
    {
      securite();
      break; // sortie du programme
    }
  }

  arret();
}
// Fin du Programme principal

void clignotement_LED()
{
  digitalWrite(LED, HIGH);  
  delay(100);               
  digitalWrite(LED, LOW);    
  delay(100);               
}


void arret()// sous-programme arret
{
  digitalWrite(ouverture, LOW);
  digitalWrite(fermeture, LOW); // arrêt du moteur
  digitalWrite(LED, LOW); // extinction de la LED
}


void securite() // sous-programme securite
{

  digitalWrite(ouverture, LOW);
  digitalWrite(fermeture, LOW); // arrêt du moteur

  tone(buzzer, 2500);
  delay(1000);
  noTone(buzzer);

  while (digitalRead(fdc_ouverture) == LOW) // rester dans la boucle tant que fdc_ouverture n'est pas activé
  {
    if (digitalRead(recepteur_IR) == HIGH)
    {
      arret();
      delay (3000); //attendre 3s
    }

    else
    {
      clignotement_LED();
      digitalWrite(ouverture, HIGH); // activation du moteur : sens ouverture
      digitalWrite(LED, HIGH); // allumage de la LED
    }
  }
  arret();
}


byte fonction_telecommande() // sous-programme fonction telecommande, retourne la touche appuyee par la telecommande
{
  byte code_recu;
  if (irReciver.decode(&results))
  {
    code_recu = results.value;
    delay(10);
    irReciver.resume(); // lire la prochaine valeur
    return (code_recu);
  }
}
